<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\CategoriesProduct;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Framework\App\ResourceConnection;
use FiloBlu\Refilo\Remote\Entity\Category;
use Zend_Db;

/**
 * Class CategoryPermissionsProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class CategoryPermissionsProvider extends BaseProvider
{
    /**
     * @var int
     */
    protected $website;
    /**
     * @var array
     */
    protected $ids;
    /**
     * @var array
     */
    protected $items;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * CategoryPermissionsProvider constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        ResourceConnection $resourceConnection
    )
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     *
     */
    public function rewind()
    {
        $permissionsByCategory = [];
        $select = $this->resourceConnection->getConnection()
            ->select()
            ->from(
                'magento_catalogpermissions'
            );
        $result = $this->resourceConnection->getConnection()->fetchAll($select, [], Zend_Db::FETCH_ASSOC);

        foreach ($result as $permission) {
            if ($permission['website_id'] == $this->website || $permission['website_id'] == null) {
                $categoryId = (int)$permission['category_id'];
                $permissionsByCategory[$categoryId]['id'] = $categoryId;
                $permissionsByCategory[$categoryId][Category::PERMISSIONS][] = $permission;
            }
        }

        $this->items = array_map(function ($item) { return new Category($item); }, $permissionsByCategory);

        if (!$this->hasReadHandler()) {
            return;
        }

        // TODO: Handle pagination
        $this->getReadHandler()->onRead($this->items, count($this->items), $this->getReadHandlerArguments());
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->website = $store->getWebsiteId();
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        $this->website = $website->getId();
        return $this;
    }
}
